/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include <string.h>

#include "internal.h"

static int
mp3file_buffer_block_next(struct MP3File *mf, int keep)
{
	int len;

	/* Keep some of the original data */
	if (keep != 0)
		memmove(mf->buf, mf->buf + MP3_BUFSIZE - keep, keep);
	
	/* Read some new data to the end of the buffer */
	if ((len = fread(mf->buf + keep, 1, MP3_BUFSIZE - keep, mf->fp)) == 0)
		return (-1);
	mf->buf_len = keep + len;

	if (mf->buf_len != MP3_BUFSIZE)
		memset(mf->buf + mf->buf_len, 0x00000000, MAD_BUFFER_GUARD);

	mad_stream_buffer(&mf->mstream, (void*)mf->buf, mf->buf_len);
	mf->mstream.error = 0;

	return (0);
}

int
mp3file_buffer_frame_next(struct MP3File *mf)
{
	for (;;) {
		/*
		 * Step 1: refill our buffer if we've run out of data.
		 */
		if ((mf->mstream.buffer == NULL) ||
		    (mf->mstream.error == MAD_ERROR_BUFLEN)) {
			/* Keep the partial last frame */
			if (mp3file_buffer_block_next(mf,
			    mf->mstream.bufend - mf->mstream.next_frame) != 0)
				return (1);
		}

		/*
		 * Step 2: try to decode our new frame.
		 */
		if (mad_frame_decode(&mf->mcurframe, &mf->mstream) == 0)
			break;

		/* Should we try again? */
		if (!MAD_RECOVERABLE(mf->mstream.error) &&
		    (mf->mstream.error != MAD_ERROR_BUFLEN))
			return (1);
	}

	/*
	 * Step 3: synthesize the frame.
	 */
	mad_synth_frame(&mf->msynth, &mf->mcurframe);

	/* Copy the sample rate and amount of channels. */
	mf->info.samplerate = mf->mcurframe.header.samplerate;
	mf->info.nchannels = MAD_NCHANNELS(&mf->mcurframe.header);

	return (0);
}
