/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include <string.h>
#include <limits.h>

#include "internal.h"

static short
mad_fixed_to_short(mad_fixed_t fixed)
{
	if (fixed >= MAD_F_ONE)
		return (SHRT_MAX);
	else if (fixed <= -MAD_F_ONE)
		return (-SHRT_MAX);
	
	return (fixed >> (MAD_F_FRACBITS - 15));
}

unsigned int
mp3file_read_short(struct MP3File *mf, short *buf, unsigned int samples)
{
	unsigned int written = 0;
	int i;

	do {
		/*
		 * Step 1: fetch the next frame if the current frame has
		 * been completely read.
		 */
		if (mf->mcursynth == 0)
			if (mp3file_buffer_frame_next(mf) != 0)
				goto done;

		/*
		 * Step 2: Copy data from the current frame.
		 */
		while ((mf->mcursynth < mf->msynth.pcm.length) &&
		    (written < samples)) {
			for (i = 0;
			    i < MAD_NCHANNELS(&mf->mcurframe.header); i++) {
				buf[written++] = mad_fixed_to_short(
				    mf->msynth.pcm.samples[i][mf->mcursynth]);
			}

			mf->mcursynth++;
			mf->curpos++;
		}

		if (mf->mcursynth == mf->msynth.pcm.length)
			mf->mcursynth = 0;
	} while (written < samples);
done:
	return (written);
}

static void
mp3file_seek_forward(struct MP3File *mf, unsigned long frames)
{
	unsigned int framesleft;

	while (frames > 0) {
		if (mf->mcursynth == 0)
			if (mp3file_buffer_frame_next(mf) != 0)
				return;

		framesleft = mf->msynth.pcm.length - mf->mcursynth;

		if (frames > framesleft) {
			frames -= framesleft;
			mf->mcursynth = 0;
			mf->curpos += framesleft;
			continue;
		} else {
			mf->mcursynth += frames;
			mf->curpos += frames;
			frames = 0;
			return;
		}
	}
}

unsigned int
mp3file_seek(struct MP3File *mf, long frames, int whence)
{
	/* Optimisation: seeking 0 bytes returns the offset */
	if (whence == SEEK_CUR && frames == 0)
		goto done;
	
	if (whence == SEEK_SET) {
		/* Rewind the file before starting */
		rewind(mf->fp);
		mf->mcursynth = 0;
		mf->curpos = 0;
	} else if (whence == SEEK_END) {
		/* XXX TO BE DONE */
		goto done;
	}

	if (frames > 0)
		/* Skip a bit */
		mp3file_seek_forward(mf, frames);
#if 0
	else if (frames < 0)
		/* Go back a bit */
		mp3file_seek_backward(mf, -frames);
#endif
done:
	return (mf->curpos);
}
