/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include <stdio.h>
#include <stdlib.h>

#include "internal.h"

#define mp3file_alloc()	calloc(1, sizeof(struct MP3File))

static void
mp3file_dealloc(struct MP3File *mf)
{
	unsigned long i;

	/* Deallocate the ID3 tags */
	for (i = 0; i < mf->info.ntags; i++)
		free(mf->info.tags[i].value);
	free(mf->info.tags);
	
	/* Base structure itself */
	free(mf);
}


struct MP3File *
mp3file_open(const char *path)
{
	FILE *fp;
	struct MP3File *ret;

	/* Try to open the file */
	fp = fopen(path, "rb");
	if (fp == NULL)
		return (NULL);
	
	/* fpopen() could handle this as well */
	ret = mp3file_fpopen(fp);
	if (ret == NULL)
		fclose(fp);
	
	return (ret);
}

struct MP3File *
mp3file_fpopen(FILE *fp)
{
	struct MP3File *mf;

	if (mp3file_match(fp) != 0)
		return (NULL);
	
	mf = mp3file_alloc();
	if (mf == NULL)
		return (NULL);
	
	mf->fp = fp;

	/*
	 * File is an MP3 audio file, so try to decode the tags,
	 * calculate the file length, etc.
	 */
	mp3file_readtags(mf);

	mad_stream_init(&mf->mstream);
	mad_frame_init(&mf->mcurframe);
	mad_synth_init(&mf->msynth);
	mf->curpos = 0;

	return (mf);
}

void
mp3file_close(struct MP3File *mf)
{
	fclose(mf->fp);
	mp3file_dealloc(mf);
}

struct MP3Info *
mp3file_getinfo(struct MP3File *mf)
{
	return (&mf->info);
}
