/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#ifndef _MP3FILE_H_
#define _MP3FILE_H_

#include <stdio.h>

struct MP3File;

struct MP3Tag {
	char		type[4];
	char		*value;
};

struct MP3Info {
	/*
	 * Data format
	 */
	unsigned long	samplerate;
	unsigned long	nchannels;
	unsigned long	nframes;

	/*
	 * ID3 tags
	 */
	unsigned long	ntags;
	struct MP3Tag	*tags;
};

/*
 * Opening and closing files
 */
struct MP3File	*mp3file_open(const char *path);
struct MP3File	*mp3file_fpopen(FILE *fd);
void		mp3file_close(struct MP3File *mf);

/*
 * File data
 */
unsigned int	mp3file_read_short(struct MP3File *mf, short *buf,
		    unsigned int samples);
unsigned int	mp3file_seek(struct MP3File *mf, long frames,
		    int whence);

/*
 * Metadata
 */
struct MP3Info	*mp3file_getinfo(struct MP3File *mf);

#endif /* !_MP3FILE_H_ */
