/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "internal.h"

#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <id3tag.h>

/*
 * This file adds bindings to libid3tag. This library has the ability to
 * parse ID3 tags from songs. Internally, it works with FILE* to read
 * and write data to the file. The API, however, forces you to pass file
 * descriptors. Our only way to work around this bogus limitation is
 * saving the position of the file descriptor to make sure our FILE*
 * structure doesn't hose itself.
 */

void
mp3file_readtags(struct MP3File *mf)
{
	int fd, i;
	off_t orig_off;
	struct id3_file *id3f;
	struct id3_tag *tag;
	id3_ucs4_t const *str;

	/* Duplicate the current filehandle */
	fd = dup(fileno(mf->fp));
	if (fd < 0)
		return;
	
	/* Backing up the original offset */
	orig_off = lseek(fd, 0, SEEK_CUR);

	id3f = id3_file_fdopen(fd, ID3_FILE_MODE_READONLY);
	if (id3f == NULL)
		goto bad;
	
	/* Here we go */
	tag = id3_file_tag(id3f);
	if (tag == NULL)
		goto done;

	mf->info.tags = calloc(tag->nframes, sizeof(struct MP3Tag));
	if (mf->info.tags == NULL)
		goto done;
	mf->info.ntags = tag->nframes;

	for (i = 0; i < tag->nframes; i++) {
		strncpy(mf->info.tags[i].type, tag->frames[i]->id, 4);
		if (id3_field_getnstrings(&tag->frames[i]->fields[1]) != 0) {
			str = id3_field_getstrings(&tag->frames[i]->fields[1], 0);
			if (str != NULL)
				mf->info.tags[i].value = (char*)id3_ucs4_latin1duplicate(str);
		}
	}
	
done:
	id3_file_close(id3f);
bad:
	lseek(fd, orig_off, SEEK_SET);
}
