/*
 * Copyright (c) 2001-2004 Bertrand Petit
 *
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */
/**
 * @file audio/mp3/mp3file.c
 */

#include <glib.h>

#include <string.h>

#include <mp3file.h>

#include <audio/mp3/driver.h>
#include <audio/output/output.h>

static void
mp3_update(struct audio_file *fd)
{
	struct MP3File *mfp = fd->drv_data;
	struct MP3Info *minf;
	
	minf = mp3file_getinfo(mfp);
	
	fd->srate = minf->samplerate;
	fd->channels = minf->nchannels;
	fd->time_cur = mp3file_seek(mfp, 0, SEEK_CUR) / minf->samplerate;
}

int
mp3_open(struct audio_file *fd)
{
	int i;
	struct MP3File *mfp;
	struct MP3Info *minf;

	mfp = mp3file_fpopen(fd->fp);
	if (mfp == NULL)
		return (-1);

	fd->drv_data = (void *)mfp;

	/* Copy the file information */
	minf = mp3file_getinfo(mfp);

	for (i = 0; i < minf->ntags; i++) {
		if (strncmp("TPE1", minf->tags[i].type, 4) == 0) {
			fd->tag.artist = g_strdup(minf->tags[i].value);
		} else if (strncmp("TIT2", minf->tags[i].type, 4) == 0) {
			fd->tag.title = g_strdup(minf->tags[i].value);
		} else if (strncmp("TALB", minf->tags[i].type, 4) == 0) {
			fd->tag.album = g_strdup(minf->tags[i].value);
		}
	}

	return (0);
}

void
mp3_close(struct audio_file *fd)
{
	struct MP3File *mfp = fd->drv_data;
	
	mp3file_close(mfp);

	/* fd->fp already closed by mp3file_close */
	fd->fp = NULL;
}

size_t
mp3_read(struct audio_file *fd, void *buf)
{
	struct MP3File *mfp = fd->drv_data;
	int ret;
#if G_BYTE_ORDER != G_LITTLE_ENDIAN
	int i;
#endif /* G_BYTE_ORDER != G_LITTLE_ENDIAN */

	ret = mp3file_read_short(mfp, buf, AUDIO_OUTPUT_BUFLEN / sizeof(short));
#if G_BYTE_ORDER != G_LITTLE_ENDIAN
	/* Convert it to little endian */
	for (i = 0; i < ret; i++)
		((short *)buf)[i] = GINT16_TO_LE(((short *)buf)[i]);
#endif /* G_BYTE_ORDER != G_LITTLE_ENDIAN */

	mp3_update(fd);

	return (ret * sizeof(short));
}

void
mp3_seek(struct audio_file *fd, double len)
{
	struct MP3File *mfp = fd->drv_data;
	struct MP3Info *minf;
	
	minf = mp3file_getinfo(mfp);

	mp3file_seek(mfp, len * minf->samplerate, SEEK_CUR);
	mp3_update(fd);
}
