/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */
/**
 * @file subr.c
 */

#include <glib.h>

#ifdef G_OS_WIN32
#include <windows.h>
#endif /* G_OS_WIN32 */

#include <stdlib.h>
#include <string.h>

#include <subr.h>

#ifdef SIGWINCH
#include <pthread.h>
#endif /* SIGWINCH */

char *
h_fgetln(FILE *fp)
{
	char buf[BUFSIZ], *nl = NULL;
	GString *ret;

	/* Try to perform a first read */
	if (fgets(buf, sizeof buf, fp) == NULL)
		return (NULL);
	
	/* We have some content - now continue */
	ret = g_string_new("");
	do {
		/* See if we've found the end of the line */
		if ((nl = strpbrk(buf, "\r\n")) != NULL)
			*nl = '\0';

		/* Append it to the return value */
		g_string_append(ret, buf);
	/* Move on to the next buffer */
	} while ((nl == NULL) && (fgets(buf, sizeof buf, fp) != NULL));

	return g_string_free(ret, FALSE);
}

char *
h_realpath(const char *relpath)
{
	char rpath[PATH_MAX];

#if defined(G_OS_UNIX)
	if (realpath(relpath, rpath) == NULL)
		return (NULL);
	else
		return g_strdup(rpath);
#elif defined(G_OS_WIN32)
	LPSTR lpf;
	int ret;

	ret = GetFullPathName(filename, PATH_MAX, rpath, &lpf);
	if (ret <= 0 || ret >= PATH_MAX)
		return (NULL);
	else
		return g_strdup(rpath);
#else
#error Cannot determine real path
#endif
}

#ifdef SIGWINCH
void
h_thread_block_signals(void)
{
	sigset_t sset;

	/*
	 * Make sure we don't catch SIGWINCH'es as they are meant for
	 * the inputloop
	 */
	sigemptyset(&sset);
	sigaddset(&sset, SIGWINCH);
	pthread_sigmask(SIG_BLOCK, &sset, NULL);
}
#endif /* SIGWINCH */
