/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */
/**
 * @file subr.h
 */

#ifndef _SUBR_H_
#define _SUBR_H_

#include <glib.h>

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>

/**
 * @brief Read a line from a file descriptor and return it in an
 *        allocated null terminated buffer.
 */
char	*h_fgetln(FILE *fp);

/**
 * @brief Resolve a relative pathname to contain an absolute one.
 */
char	*h_realpath(const char *relpath);

/**
 * @brief Remove signals from threads that are needed for other
 *        purposes.
 */
#ifdef SIGWINCH
void	h_thread_block_signals(void);
#else /* !SIGWINCH */
#define h_thread_block_signals()
#endif /* SIGWINCH */

#ifndef g_slice_new
#define g_slice_new(type) g_malloc(sizeof(type))
#define g_slice_new0(type) g_malloc0(sizeof(type))
#define g_slice_free(type, addr) g_free(addr)
#endif /* !g_slice_new */

#ifdef G_OS_WIN32
#define lstat(path,sb) stat(path,sb)
#endif /* G_OS_WIN32 */

#endif /* !_SUBR_H_ */
