/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */
/**
 * @file vfs/modules/vfs_win32.h
 */

/**
 * @brief Open the virtual root directory for Microsoft Windows. This
 *        operating system does not have a single root directory.
 */
static int
vfs_win32_root_open(struct vfs_ent *ve, int isdir)
{
	return strcmp(ve->filename, PATH_ROOT);
}

/**
 * @brief Fill our virtual root directory with a node for each available
 *        storage device.
 */
static int
vfs_win32_root_populate(struct vfs_ent *ve)
{
	struct vfs_ent *new;
	char i, drivepath[] = "A:";
	DWORD drives = GetLogicalDrives();

	for (i = 'A'; i <= 'Z'; i++) {
		if (drives & (1 << (i - 'A'))) {
			/* Generate a pathname */
			drivepath[0] = i;

			/* Open the drive */
			new = vfs_open(drivepath, NULL);
			if (new != NULL)
				TAILQ_INSERT_TAIL(&ve->population, new, next);
		}
	}

	return (0);
}
